
module Pod
    class Podfile
        module DSL

            # 从网络获取 pod 列表，并插入 podfile 中
            #
            # 会以 podfile 同级中的 integration.yml 文件为参数，请求内容
            # (yml 中读取 top level 的 history_id
            #
            def bd_fetch_pod_list(with_patch_feature = true)
                # 默认带 patch 功能
                if with_patch_feature
                    bd_use_patch_file_begin!
                    BD::FetchPodList.new(self).main
                    bd_use_patch_file_end!
                else
                    BD::FetchPodList.new(self).main
                end
            end

        end
    end
end


module BD
  
    # 支持集成区，从网络获取 pod 列表
    class FetchPodList < PodfileEnvironment
  
      private 

      CONFIG_NAME = "integration.yml"

      def config_path # [Pathname]
        podfile.defined_in_file.dirname + CONFIG_NAME
      end

      def get_id # [String]
        raise "Cannot find #{CONFIG_NAME} file in the same directory with Podfile" unless config_path.file?
        text = File.read(config_path)
        hash = Pod::YAMLHelper.load_string(text)
        id = hash["history_id"]
        raise "No history_id found in #{CONFIG_NAME}" if id.nil?
        id
      end
  
      def generated_podlist_file_path # [String]
        @podlist_path ||= Tool::Path.config.sandbox_root + "PodsSummary"
        @podlist_path
      end
  
      def generate_url_for_id(ids) # [string]
        url = "https://ios.bytedance.net/api/v1/mergeRequest/components_list"
        url += "?mrUpdateId=#{ids}"
      end
  
      def fetch_pod_list(ids) # [Hash]
        puts "Fetching pod list: #{ids} ..."
        def fetch(url)
          begin
            url_str = URI.parse(url)
            return Net::HTTP.get_response(url_str)
          rescue Exception => ex
            puts "[!] Pod 列表请求失败. (#{url})".red
            raise ex
          end
        end
  
        def convertToPodfileFormat(response) # [Hash]
          targetNameArray = Array.new
          targetHash = Hash.new
          response["data"]["components"].each do |pod|
            #先判断是否有新增target，因为平台中组件不会重复
            pod["app_struct"].each_key do |targetName|
              unless targetNameArray.include?(targetName)
                targetNameArray << targetName
                # 创建对应的target
                targetDef = Hash.new
                targetDef["name"] = targetName
                targetDef["components"] = Array.new
                # 添加到target中
                targetHash[targetName] = targetDef
              end
              # 创建pod的定义
              pod_def = Hash.new
              pod_def["name"] = pod["name"]
              pod_def["version"] = pod["version"]
              if pod["app_struct"][targetName].size != 0
                pod_def["subspecs"] = pod["app_struct"][targetName]
              end
              targetHash[targetName]["components"] << pod_def
            end
          end
    
          project_adapt = targetHash.values
          completeData = Hash.new
          completeData["projects"] = project_adapt
          completeData
        end
  
        config_json = begin
          url = generate_url_for_id(ids)
          respose = fetch(url)
          json = JSON.parse(respose.body)
          if json["errno"] != 200
            raise "[!] Pod 列表请求错误：#{json["errmsg"]}"
          end
          convertToPodfileFormat(json)
        end
      end
  
      def self.format_additional_parameters(pod_json) # [Array<String>]
        pod_json.map do |key,value|
          next ":#{key} => #{value}" if key != 'name' && key != 'version'
          next nil
        end.reject(&:nil?)
      end
  
      # convert the json to podfile format content (ruby code)
      def convertListToPodfileContent(info)
        podfile_content = ""
  
        info['projects'].each do |project|
          podfile_content += "target '#{project['name']}' do\n"
          project['components'].each do |podInfo|
            parameters = ["'#{podInfo['name']}'", "'#{podInfo['version']}'"]
            parameters += FetchPodList.format_additional_parameters(podInfo)
            podfile_content += "    pod_binary #{parameters.join(", ")}\n"
          end
          podfile_content += "end\n\n"
        end
        podfile_content
      end
  
      # convert the json to human readable content (text)
      def convertListToHumanReadableText(info)
        content = ""
  
        info['projects'].each do |project|
          content += "#{project['name']}:\n"
          project['components'].each do |podInfo|
            parameters = [podInfo['name'].ljust(18), "#{podInfo['version']}"]
            parameters += FetchPodList.format_additional_parameters(podInfo)
            content += "    - #{parameters.join(", ")}\n"
          end
          content += "\n"
        end
        content
      end
  
  
      # add the pod in list to podfile
      def addToPodfile(info)
        def adaptPodinfoToHashArray(podInfo)
          podInfo.map do |key,value|
            next Hash[key.to_sym => value] if key != 'name' && key != 'version'
            next nil
          end.reject(&:nil?)
        end
  
        podfile_content = ""
  
        info['projects'].each do |project|
          # taget TARGET_NAME do 
          podfile.target project['name'] do
            project['components'].each do |podInfo|
              value = adaptPodinfoToHashArray(podInfo)
              # pod_binary POD_NAME, VERSION
              podfile.pod_binary podInfo['name'],podInfo['version'], *value
            end
          # end
          end
        end
      end
  
  
      public
  
      def main
        podfile.plugin 'cocoapods-bytedance-newIntegrate'
        pod_list_id = get_id
        config = fetch_pod_list(pod_list_id)
        addToPodfile(config)
  
        # write content to file, for human reading
        genereted_pod_content = convertIdToPodsSummary(pod_list_id, config)
        list_file_path = generated_podlist_file_path
        File.write(list_file_path, genereted_pod_content)
        $BD_GENERATED_POD_LIST_FILE = list_file_path
  
      end
  
      def convertIdToPodfileContent(id, config_json = nil) # [String]
        podfile_content = ""
        podfile_content += "# This file is generated automatically. DO NOT MODIFY MANUALY!\n"
        podfile_content += "# Source ID: #{id} \n\n"
        podfile_content += convertListToPodfileContent(config_json || fetch_pod_list(id))
        podfile_content += "# This file is generated automatically. DO NOT MODIFY MANUALY!\n"
        return podfile_content
      end
  
      def convertIdToPodsSummary(id, config_json = nil) # [String]
        podfile_content = ""
        podfile_content += "# This file is generated automatically. DO NOT MODIFY MANUALY!\n"
        podfile_content += "# This file is just for human reading.\n"
        podfile_content += "#\n"
        podfile_content += "# Source ID: #{id} \n"
        podfile_content += "# These pods are imported by `bd_fetch_pod_list XXXXX` in podfile\n\n"
        podfile_content += convertListToHumanReadableText(config_json||fetch_pod_list(id))
        podfile_content += "# Equivalent Podfile content: \n\n"
        podfile_content += begin 
          text = convertListToPodfileContent(config_json || fetch_pod_list(id))
          text.split("\n").map { |line| "#    " + line }.join("\n")
        end
        return podfile_content
      end
    end
  
  end
  



Pod::HooksManager.register('cocoapods-bytedance-newIntegrate', :pre_install) do |installer_context|

  Dir.chdir(File::dirname(installer_context.sandbox_root))

  if !ENV['WORKSPACE']
    cmd = "
    root=`git rev-parse --show-toplevel`
    cd $root
    if [[ -d .git ]]; then
      cd .git
      [[ ! -d hooks ]] && mkdir hooks 
      cd hooks
      rm -rf post-checkout
      rm -rf post-checkout.sample
      rm -rf post-pull
      rm -rf post-pull.sample
      rm -rf post-merge
      rm -rf post-merge.sample
      curl -o post-checkout 'http://tosv.byted.org/obj/iosbinary/bd_pod_extentions/6.3.0/post-checkout'
      chmod 777 post-checkout
      curl -o post-pull 'http://tosv.byted.org/obj/iosbinary/bd_pod_extentions/6.3.0/post-pull'
      chmod 777 post-pull
      curl -o post-merge 'http://tosv.byted.org/obj/iosbinary/bd_pod_extentions/6.3.0/post-merge'
      chmod 777 post-merge
      curl -o newIntegrate.sh 'http://tosv.byted.org/obj/iosbinary/bd_pod_extentions/6.3.0/newIntegrate.sh'
      bash newIntegrate.sh
    fi
    "
    system(cmd)
    file =`
    root=\`git rev-parse --show-toplevel\`
    cd $root
    cd .git/hooks
    echo \`pwd\`"/Podfile_Map"
    `
    Podfile_Map = file.strip

    module Pod
      class Project < Xcodeproj::Project
        alias bd_add_podfile add_podfile
        def add_podfile(podfile_path)
          bd_add_podfile($BD_GENERATED_POD_LIST_FILE) if File::file?($BD_GENERATED_POD_LIST_FILE)
          # bd_add_podfile(Podfile_Map) if File::file?(Podfile_Map)
          bd_add_podfile($Platform_Config) if File::file?($Platform_Config)
          bd_add_podfile(File::expand_path('Podfile_Patch')) if File::file?('Podfile_Patch')
          bd_add_podfile(podfile_path)
        end
      end
    end
  end
end
